/*----------------------------------------------------------------------------
 *
 *   Copyright (C) 2021 Antonio Augusto Alves Junior
 *
 *   This file is part of RandomIterator.
 *
 *   RandomIterator is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   RandomIterator is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with RandomIterator.  If not, see <http://www.gnu.org/licenses/>.
 *
 *---------------------------------------------------------------------------*/
/*
 * example.cpp
 *
 *  Created on: 11/05/2021
 *      Author: Antonio Augusto Alves Junior
 */

/**
 * \example example.cpp
 *
 */



#include <iostream>
#include <random>
#include <vector>
#include <string>
#include <random_iterator/Stream.hpp>

//command line
#define TCLAP_SETBASE_ZERO 1
#include <tclap/CmdLine.h>

//set a global seed
static const uint64_t default_seed= 0x548c9decbce65295  ;




int main(int argv, char** argc)
{
	uint64_t seed   = default_seed;
	uint32_t stream = 0;
    size_t nentries = 0;

	try {

		TCLAP::CmdLine cmd("Command line arguments for ", '=');

		TCLAP::ValueArg<size_t> NEntriesArg("n", "nentries", "Number of entries.", false, 0, "size_t");
		cmd.add(NEntriesArg);

		TCLAP::ValueArg<uint64_t> SeedArg("s", "seed", "RNG seed.", false, default_seed, "uint64_t");
		cmd.add(SeedArg);

		TCLAP::ValueArg<uint32_t> StreamArg("S", "stream", "Stream number.", false, 0, "uint32_t") ;
		cmd.add(StreamArg);

		// Parse the argv array.
		cmd.parse(argv, argc);

		// Get the value parsed by each arg.
		seed      = SeedArg.getValue();
        stream    = StreamArg.getValue();
        nentries  = NEntriesArg.getValue();
	}
	catch (TCLAP::ArgException &e)  {
		std::cerr << "error: " << e.error() << " for arg " << e.argId()	<< std::endl;
	}

	random_iterator::philox RNG(seed);

	std::uniform_real_distribution<double> dist(0.0, 10.0);

	auto rng_stream = random_iterator::make_stream( dist, RNG, stream);

	//histogram as a simple vector
	std::vector<size_t> histogram(10);

	//fill histogram
    do {

    	++histogram[ size_t(rng_stream()) ];

    } while( ( --nentries ) > 0 );

   //print histogram nicely
   size_t i =0;
   size_t maxe = *std::max_element(histogram.begin(), histogram.end());
   std::cout <<maxe << std::endl ;
   std::cout << "--------"<< std::string(50 , '-') << std::endl ;

   for(auto n: histogram){

	   double bin = 50.0*double(n)/maxe;
	   std::cout << "["<< i<<", "<< ++i << ") "<<
			   std::string( bin, '=') << ": "<< n <<std::endl;
   }
   std::cout << "--------"<< std::string(50, '-') << std::endl ;

}
